#include <QSettings>
#include <QTimer>
#include <QKeyEvent>
#include <QPainter>
#include <QLinearGradient>
#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "dlgsleep.h"

MainWindow* g_pMainWindow = NULL;

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    g_pMainWindow = this;

    _aliveTimer = new QTimer(this);
    connect(_aliveTimer, SIGNAL(timeout()), this, SLOT(aliveTimeout()));
    keepAlive();

    ui->setupUi(this);

#ifdef QT_ARCH_ARM
    showFullScreen();
#else
    setGeometry(0,0,600,800);
#endif

    int maxDimension = height();
    if (maxDimension < width())
        maxDimension = width();

    if (maxDimension < 810)
    {
        // STATE: should be Kindle 2
        _isKindle2 = true;

        shrinkFont(ui->cbHelloWorld);
    }

    _startupCounter = 0;
    _backgroundCounter = 0;
    _background = 0;

    ui->cbHelloWorld->setGeometry(0, 0, 150, 30);

    _movementTimer = new QTimer(this);
    connect(_movementTimer, SIGNAL(timeout()), this, SLOT(movementTimeout()));
    _movementTimer->start(5000);
}

MainWindow::~MainWindow()
{
    delete ui;

    g_pMainWindow = NULL;
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void MainWindow::paintEvent(QPaintEvent* e)
{
    Q_UNUSED(e);

    QPainter painter(this);

    int whichBackground = _background % 3;

    if (_startupCounter < 2)
        whichBackground = _startupCounter++;

    if (whichBackground == 0)
        painter.fillRect(rect(), Qt::white);
    else if (whichBackground == 1)
        painter.fillRect(rect(), Qt::black);
    else
    {
        QLinearGradient gradient(rect().topLeft(), rect().bottomLeft());
            gradient.setColorAt(0, Qt::black);
            gradient.setColorAt(1, Qt::white);

        painter.fillRect(rect(), gradient);
    }
}

void MainWindow::keepAlive()
{
    _aliveTimer->stop();
    _aliveTimer->start(5 * 60 * 1000 - 10000); // 4 minutes 50 seconds, show the sleep warning
}

void MainWindow::aliveTimeout()
{
    if (_dlgSleep != NULL)
        delete _dlgSleep;

    _dlgSleep = new dlgSleep(this);

    _dlgSleep->exec();

    g_pMainWindow->keepAlive();
}

bool MainWindow::isKindle2() const
{
    return _isKindle2;
}

void MainWindow::shrinkFont(QWidget* w)
{
    QFont f = w->font();
    f.setPointSizeF(f.pointSizeF() / 2.0f);
    w->setFont(f);
}

void MainWindow::setSleepDialog(dlgSleep* dlgSleep)
{
    _dlgSleep = dlgSleep;
}

int MainWindow::translateNumberKey(int key)
{
    switch(key)
    {
    default:
        return key - Qt::Key_0;
    case Qt::Key_Q:
    case Qt::Key_Exclam:
        return 1;
    case Qt::Key_W:
    case Qt::Key_At:
        return 2;
    case Qt::Key_E:
    case Qt::Key_NumberSign:
        return 3;
    case Qt::Key_R:
    case Qt::Key_Dollar:
        return 4;
    case Qt::Key_T:
    case Qt::Key_Percent:
        return 5;
    case Qt::Key_Y:
    case Qt::Key_AsciiCircum:
        return 6;
    case Qt::Key_U:
    case Qt::Key_Ampersand:
        return 7;
    case Qt::Key_I:
    case Qt::Key_Asterisk:
        return 8;
    case Qt::Key_O:
    case Qt::Key_ParenLeft:
        return 9;
    case Qt::Key_P:
        return 0;
    }
}

void MainWindow::keyPressEvent(QKeyEvent* e)
{
    int number;

    keepAlive();

//    qDebug("key %d, modifiers %d (%s)", (int)e->key(), (int)e->modifiers(), ((e->modifiers() & Qt::ShiftModifier) ? "SHIFT" : "no shift"));

    if (e->modifiers() & Qt::ShiftModifier)
    {
        switch(e->key())
        {
        default:
            return;
        case Qt::Key_Up:
            // fiveway moving up with shift key down
            return;
        case Qt::Key_Down:
            // fiveway moving down with shift key down
            return;
        case Qt::Key_Left:
            // fiveway moving left with shift key down
            return;
        case Qt::Key_Right:
            // fiveway moving right with voidshift key down
            return;
        case Qt::Key_Q:
        case Qt::Key_W:
        case Qt::Key_E:
        case Qt::Key_R:
        case Qt::Key_T:
        case Qt::Key_Y:
        case Qt::Key_U:
        case Qt::Key_I:
        case Qt::Key_O:
        case Qt::Key_P:
        case Qt::Key_1:
        case Qt::Key_2:
        case Qt::Key_3:
        case Qt::Key_4:
        case Qt::Key_5:
        case Qt::Key_6:
        case Qt::Key_7:
        case Qt::Key_8:
        case Qt::Key_9:
        case Qt::Key_0:
        case Qt::Key_Exclam:
        case Qt::Key_At:
        case Qt::Key_NumberSign:
        case Qt::Key_Dollar:
        case Qt::Key_Percent:
        case Qt::Key_AsciiCircum:
        case Qt::Key_Ampersand:
        case Qt::Key_Asterisk:
        case Qt::Key_ParenLeft:
            number = translateNumberKey(e->key());

            // handle number with shift key down
            return;
        }
    }

    switch(e->key())
    {
    default:
        break;
    case Qt::Key_F1:
        // Kindle 2 dimensions
        showNormal();
        setGeometry(0, 0, 600, 800);
        break;
        bool loadGame();
    case Qt::Key_F2:
        // Kindle DX dimensions
        showNormal();
        setGeometry(0, 0, 824, 1200);
        break;
    case Qt::Key_F3:
        // fullscreen mode
        showFullScreen();
        break;
    case Qt::Key_Home:
        // save and quit
        saveGame();
        QCoreApplication::quit();
        break;
    case Qt::Key_Up:
        // fiveway moving up
        break;
    case Qt::Key_Down:
        // fiveway moving down
        break;
    case Qt::Key_Left:
        // fiveway moving left
        break;
    case Qt::Key_Right:
        // fiveway moving right
        break;

    case Qt::Key_1:
    case Qt::Key_2:
    case Qt::Key_3:
    case Qt::Key_4:
    case Qt::Key_5:
    case Qt::Key_6:
    case Qt::Key_7:
    case Qt::Key_8:
    case Qt::Key_9:
    case Qt::Key_0:
    case Qt::Key_Exclam:
    case Qt::Key_At:
    case Qt::Key_NumberSign:
    case Qt::Key_Dollar:
    case Qt::Key_Percent:
    case Qt::Key_AsciiCircum:
    case Qt::Key_Ampersand:
    case Qt::Key_Asterisk:
    case Qt::Key_ParenLeft:
        number = translateNumberKey(e->key());

        // handle number
        break;
    }
}

bool MainWindow::loadGame()
{
    QSettings settings("/mnt/us/system-kindleqt/etc/KindleSampleQt.ini", QSettings::IniFormat);

/*
    SAMPLE IMPLEMENTATION FROM SUDOKU GAME:

    QVariant v = settings.value("AutoFill", QVariant(0));

    if (v.isNull() || (v.type() != QVariant::String))
        _autoFill = false;
    else
        _autoFill = (v.toInt() != 0);

    v = settings.value("InvertScratchColors", QVariant(0));

    if (v.isNull() || (v.type() != QVariant::String))
        _invertScratchColors = false;
    else
        _invertScratchColors = (v.toInt() != 0);

    v = settings.value("Board", QVariant());

    if (v.isNull() || (v.type() != QVariant::String))
    {
        newGame();
        return;
    }

    SudokuBoard* newBoard = new SudokuBoard(v.toString(), this);

    if (newBoard->error())
    {
        delete newBoard;
        newGame();
        return;
    }

    setBoard(newBoard);
*/

    return true;
}

bool MainWindow::saveGame()
{
    QSettings settings("/mnt/us/system-kindleqt/etc/KindleSampleQt.ini", QSettings::IniFormat);

/*
    SAMPLE IMPLEMENTATION FROM SUDOKU GAME:

    settings.setValue("AutoFill", QVariant(_autoFill ? 1 : 0));
    settings.setValue("InvertScratchColors", QVariant(_invertScratchColors ? 1 : 0));
    settings.setValue("Board", QVariant(_board->toString()));

    settings.sync();

    _dirty = false;
*/

    return true;
}

void MainWindow::movementTimeout()
{
    ui->cbHelloWorld->move(qrand() % (width() - ui->cbHelloWorld->width()), qrand() % (height() - ui->cbHelloWorld->height()));

    if (_backgroundCounter >= 10)
    {
        _backgroundCounter = 0;

        _background++;

        ui->cbHelloWorld->setText("Hello World");
    }
    else
        _backgroundCounter++;

    update();
}

void MainWindow::on_cbHelloWorld_clicked()
{
    ui->cbHelloWorld->setText("Ouch!");
}
